import 'dart:convert';

class ApiResponse<T> {
  static withError<T>(Object error) {
    return ApiErrorResponse<T>(error?.toString() ?? 'Unknown error');
  }

  static withResponse<T>(int statusCode, dynamic rawBody, T parsedBody) {
    if (statusCode == 200) {
      if (parsedBody == null || statusCode == 204) {
        return ApiEmptyResponse<T>();
      } else {
        return ApiSuccessResponse<T>(parsedBody);
      }
    } else {
      var error;
      try {
        var j = json.decode(rawBody);
        error = j['error'] ?? j['message'];
      } catch (e, s) {
        print(s);
      }
      return ApiErrorResponse<T>(error ?? rawBody ?? 'Unknown error');
    }
  }
}

class ApiEmptyResponse<T> extends ApiResponse<T> {}

class ApiSuccessResponse<T> extends ApiResponse<T> {
  T body;

  ApiSuccessResponse(this.body);
}

class ApiErrorResponse<T> extends ApiResponse<T> {
  String errorMessage;

  ApiErrorResponse(this.errorMessage);
}
