class Resource<T> {
  Resource(this.status, this.data, this.message);

  final ResourceStatus status;
  final T data;
  final String message;

  bool isLoading() => status == ResourceStatus.loading;

  bool isEmpty() => status == ResourceStatus.empty;

  bool isError() => status == ResourceStatus.error;

  factory Resource.success(T data) {
    return Resource(ResourceStatus.success, data, null);
  }

  factory Resource.error(dynamic error, [T data]) {
    return Resource(ResourceStatus.error, data, error.toString());
  }

  factory Resource.loading([T data]) {
    return Resource(ResourceStatus.loading, data, null);
  }

  factory Resource.empty() {
    return Resource(ResourceStatus.empty, null, null);
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is Resource &&
          runtimeType == other.runtimeType &&
          status == other.status &&
          data == other.data &&
          message == other.message;

  @override
  int get hashCode => status.hashCode ^ data.hashCode ^ message.hashCode;
}

enum ResourceStatus { success, error, loading, empty }
