import 'dart:async';

import 'package:arch_foundations/api_response.dart';
import 'package:arch_foundations/arch/network_bound_resource.dart';
import 'package:arch_foundations/resource.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  test('emits Resource.loading() when instantiated', () {
    final res = NetworkBoundResource<int, ApiResponse<int>>(
      shouldFetch: (data) => false,
      loadFromDb: () => Stream.fromIterable([1]),
      onFetchFailed: () {},
      saveCallResult: (data) {},
      createCall: () => Future.value(),
    );

    expect(res.asStream(), emits(Resource<int>.loading()));
  });

  test('emits db data if its not null when instantiated', () {
    final res = NetworkBoundResource<int, ApiResponse<int>>(
      shouldFetch: (data) => false,
      loadFromDb: () => Stream.fromIterable([1]),
      onFetchFailed: () {},
      saveCallResult: (data) {},
      createCall: () => Future.value(),
    );

    expect(
      res.asStream(),
      emitsInOrder([
        Resource<int>.loading(),
        Resource<int>.success(1),
      ]),
    );
  });

  test('fetches if shouldFetc is true when instantiated', () async {
    var createCallCalled = false;

    NetworkBoundResource<int, ApiResponse<int>>(
      shouldFetch: (data) => true,
      loadFromDb: () => Stream.fromIterable([1]),
      onFetchFailed: () {},
      saveCallResult: (data) {},
      createCall: () {
        createCallCalled = true;
        return Future.value();
      },
    );

    // Wait for createCall to be called
    await Future.delayed(Duration(milliseconds: 10));

    expect(createCallCalled, isTrue);
  });

  test('saves fetch result to db when successful', () async {
    var saveCalled = false;

    NetworkBoundResource<int, ApiResponse<int>>(
      shouldFetch: (data) => true,
      loadFromDb: () => Stream.fromIterable([1]),
      onFetchFailed: () {},
      saveCallResult: (data) {
        saveCalled = true;
      },
      createCall: () => Future.value(ApiSuccessResponse(null)),
    );

    // Wait for saveCallResult to be called
    await Future.delayed(Duration(milliseconds: 10));

    expect(saveCalled, isTrue);
  });

  test('calls onFetchFailed when fetch fails', () async {
    var fetchFailedCalled = false;

    NetworkBoundResource<int, ApiResponse<int>>(
      shouldFetch: (data) => true,
      loadFromDb: () => Stream.fromIterable([1]),
      onFetchFailed: () {
        fetchFailedCalled = true;
      },
      saveCallResult: (data) {},
      createCall: () => Future.value(ApiErrorResponse(null)),
    );

    // Wait for onFetchFailed to be called
    await Future.delayed(Duration(milliseconds: 10));

    expect(fetchFailedCalled, isTrue);
  });
}
